@@ -27,13 +27,20 @@ import sys
 import os
 import string
 import re
+from UserDict import UserDict
+from types import StringType
 # XXX: obsolete, should use re module
 import regsub
-import fcntl
 import random
 import mm_cfg
 import Errors
+##try:
+##    import md5
+##except ImportError:
+##    md5 = None
 
+
+
 def list_names():
     """Return the names of all lists in default list directory."""
     got = []
@@ -46,6 +53,7 @@ def list_names():
 	got.append(fn)
     return got
 
+
 # a much more naive implementation than say, Emacs's fill-paragraph!
 def wrap(text, column=70):
     """Wrap and fill the text to the specified column.
@@ -123,6 +131,7 @@ def wrap(text, column=70):
     return wrapped[:-2]
     
 
+
 def SendTextToUser(subject, text, recipient, sender, add_headers=[]):
     import Message
     msg = Message.OutgoingMessage()
@@ -131,7 +140,10 @@ def SendTextToUser(subject, text, recipient, sender, add_headers=[]):
     msg.SetBody(QuotePeriods(text))
     DeliverToUser(msg, recipient, add_headers=add_headers)
 
+
 
+_children = []
+
 def DeliverToUser(msg, recipient, add_headers=[]):
     """Use smtplib to deliver message.
 
@@ -149,7 +161,7 @@ def DeliverToUser(msg, recipient, add_headers=[]):
     # foreground*.  If the errorsto happens to be the list owner for a list
     # that is doing the send - and holding a lock - then the delivery will
     # hang pending release of the lock - deadlock.
-
+    global _children
     sender = msg.GetSender()
     try:
         msg.headers.remove('\n')
@@ -161,17 +173,19 @@ def DeliverToUser(msg, recipient, add_headers=[]):
         if i and i[-1] != '\n':
             i = i + '\n'
         msg.headers.append(i)
-
     text = string.join(msg.headers, '') + '\n' + QuotePeriods(msg.body)
     import OutgoingQueue
     queue_id = OutgoingQueue.enqueueMessage(sender, recipient, text)
-
     # If the fork (or anything the child does) fails and raises an
     # exception, e.g. due to memory exhaustion, the queue runner
     # will pick up the queue file.
     try:
-        if not os.fork():
-            # child
+        pid = os.fork()
+        if pid:
+            # in the parent
+            _children.append(pid)
+        else:
+            # in the child
             try:
                 TrySMTPDelivery(recipient,sender,text,queue_id)
             finally:
@@ -179,10 +193,25 @@ def DeliverToUser(msg, recipient, add_headers=[]):
     except:
         OutgoingQueue.deferMessage(queue_id)
 
+
+def Reap():
+    global _children
+    try:
+        pid, sts = os.waitpid(-1, os.WNOHANG)
+        try:
+            _children.remove(pid)
+        except ValueError:
+            pass
+    except os.error:
+        pass
+    return len(_children)
+
+
+
 def TrySMTPDelivery(recipient, sender, text, queue_entry):
     import socket
     import OutgoingQueue
-
+    from Logging.StampedLogger import StampedLogger
     # We need to get a modern smtplib.  The Python 1.5.1 version does not
     # support ehlo (ESMTP), but the Python 1.5.2 version does.  However, the
     # interface to conn.ehlo() has changed between Python 1.5.2beta and 1.5.2
@@ -205,6 +234,33 @@ def TrySMTPDelivery(recipient, sender, text, queue_entry):
         # here to disable/remove bad local addresses.  Currently we don't
         # know which list(s) the address is on :(
         refused = conn.sendmail(sender, recipient, text, rcpt_options=ropts)
+        #
+        # debugging
+        #
+##        l = StampedLogger('smtp-delivery', 'TrySMTPDelivery', immediate=1)
+##        sys.stdout = l
+##        try:
+##            print 'SMTP delivery:',
+##            if refused: print 'partial'
+##            else: print 'full'
+##            print 'queue_entry:', queue_entry
+##            print 'sender:', sender
+##            print 'recipient:', recipient
+##            if md5:
+##                texthash = md5.new(text).digest()
+##                x = []
+##                for c in texthash:
+##                    z = hex(ord(c))[2:]
+##                    if len(z) == 1: z = '0'+z
+##                    x.append(z)
+##                print 'md5 of text:', string.join(x, '')
+##            if refused:
+##                print 'refused:', refused
+##        finally:
+##            sys.stdout = sys.__stdout__
+        #
+        # end debugging
+        #
         conn.quit()
         defer = 0
         failure = None
@@ -268,7 +324,6 @@ def TrySMTPDelivery(recipient, sender, text, queue_entry):
         t, v = failure[0], failure[1]
         # XXX Here may be the place to get the failure info back to the list
         # object, so it can disable the recipient, etc.  But how?
-        from Logging.StampedLogger import StampedLogger
         l = StampedLogger("smtp-failures", "TrySMTPDelivery", immediate=1)
         l.write("To %s:\n" % recipient)
         l.write("\t %s" % t)
@@ -280,6 +335,7 @@ def TrySMTPDelivery(recipient, sender, text, queue_entry):
         l.flush()
 
 
+
 def QuotePeriods(text):
     return string.join(string.split(text, '\n.\n'), '\n .\n')
 
@@ -306,6 +362,7 @@ def ValidateEmail(str):
 	raise Errors.MMBadEmailError
 
 
+
 # User J. Person <person@allusers.com>
 _addrcre1 = re.compile('<(.*)>')
 # person@allusers.com (User J. Person)
@@ -347,6 +404,7 @@ def ParseAddrs(addresses):
     return map(string.strip, parsed)
 
 
+
 def GetPathPieces(path):
     l = string.split(path, '/')
     try:
@@ -356,6 +414,7 @@ def GetPathPieces(path):
 	pass
     return l
 
+
 nesting_level = None
 def GetNestingLevel():
   global nesting_level
@@ -369,6 +428,8 @@ def GetNestingLevel():
       nesting_level = 0
   return nesting_level
 
+
+
 def MakeDirTree(path, perms=0775, verbose=0):
     made_part = '/'
     path_parts = GetPathPieces(path)
@@ -387,6 +448,8 @@ def MakeDirTree(path, perms=0775, verbose=0):
 	    if verbose:
 		print 'made directory: ', madepart
   
+
+
 # This takes an email address, and returns a tuple containing (user,host)
 def ParseEmail(email):
     user = None
@@ -409,7 +472,7 @@ def LCDomain(addr):
     return addr[:atind] + '@' + string.lower(addr[atind + 1:])
 
 
-
+
 # Return 1 if the 2 addresses match.  0 otherwise.
 # Might also want to match if there's any common domain name...
 # There's password protection anyway.
@@ -434,7 +497,7 @@ def AddressesMatch(addr1, addr2):
     return 1
 
 
-
+
 def GetPossibleMatchingAddrs(name):
     """returns a sorted list of addresses that could possibly match
     a given name.
@@ -454,6 +517,7 @@ def GetPossibleMatchingAddrs(name):
     return res
 
 
+
 def List2Dict(list):
     """List2Dict returns a dict keyed by the entries in the list
     passed to it."""
@@ -487,6 +551,7 @@ def FindMatchingAddresses(name, dict, *dicts):
     return res
 
   
+
 def GetRandomSeed():
     chr1 = int(random.random() * 52)
     chr2 = int(random.random() * 52)
@@ -507,6 +572,7 @@ def MakeRandomPassword(length=4):
     return password
 
 
+
 def SnarfMessage(msg):
     if msg.unixfrom:
 	text = msg.unixfrom + string.join(msg.headers, '') + '\n' + msg.body
@@ -515,6 +581,7 @@ def SnarfMessage(msg):
     return (msg.GetSender(), text) 
 
 
+
 def QuoteHyperChars(str):
     arr = regsub.splitx(str, '[<>"&]')
     i = 1
@@ -530,9 +597,10 @@ def QuoteHyperChars(str):
 	i = i + 2
     return string.join(arr, '')
 
+
+
 # Just changing these two functions should be enough to control the way
 # that email address obscuring is handled.
-
 def ObscureEmail(addr, for_text=0):
     """Make email address unrecognizable to web spiders, but invertable.
 
@@ -550,6 +618,7 @@ def UnobscureEmail(addr):
     return string.replace(addr, "__at__", "@")
 
 
+
 def map_maillists(func, names=None, unlock=None, verbose=0):
     """Apply function (of one argument) to all list objs in turn.
 
@@ -561,22 +630,27 @@ def map_maillists(func, names=None, unlock=None, verbose=0):
     instantiated, CR when instantiation is complete, and result of
     application as it shows."""
     from MailList import MailList
-    if names == None: names = list_names()
+    if names == None:
+        names = list_names()
     got = []
     for i in names:
-	if verbose: print i,
+	if verbose:
+            print i,
 	l = MailList(i)
-	if verbose: print
+	if verbose:
+            print
 	if unlock and l.Locked():
 	    l.Unlock()
 	got.append(apply(func, (l,)))
-	if verbose: print got[-1]
+	if verbose:
+            print got[-1]
 	if not unlock:
 	    l.Unlock()
 	del l
     return got
 
 
+
 def chunkify(members, chunksize=None):
      """
      return a list of lists of members
@@ -594,6 +668,28 @@ def chunkify(members, chunksize=None):
      return res
 
 
+
+class SafeDict(UserDict):
+    """Dictionary which returns a default value for unknown keys.
+
+    This is used in maketext so that editing templates is a bit more robust.
+    """
+    def __init__(self, d):
+        # optional initial dictionary is a Python 1.5.2-ism.  Do it this way
+        # for portability
+        UserDict.__init__(self)
+        self.update(d)
+
+    def __getitem__(self, key):
+        try:
+            return self.data[key]
+        except KeyError:
+            if type(key) == StringType:
+                return '%('+key+')s'
+            else:
+                return '<Missing key: %s>' % `key`
+
+
 def maketext(templatefile, dict, raw=0):
     """Make some text from a template file.
 
@@ -605,12 +701,13 @@ def maketext(templatefile, dict, raw=0):
     fp = open(file)
     template = fp.read()
     fp.close()
+    text = template % SafeDict(dict)
     if raw:
-        return template % dict
-    return wrap(template % dict)
+        return text
+    return wrap(text)
 
 
-#
+
 # given an IncomingMessage object,
 # test for administrivia (eg subscribe, unsubscribe, etc).
 # the test must be a good guess -- messages that return true
@@ -681,6 +778,7 @@ def reraise(exc=None, val=None):
     raise exc, val, sys.exc_info()[2]
 
 
+
 def mkdir(dir, mode=02775):
     """Wraps os.mkdir() in a umask saving try/finally.
 Two differences from os.mkdir():
@@ -693,6 +791,8 @@ Two differences from os.mkdir():
     finally:
         os.umask(ou)
 
+
+
 def open_ex(filename, mode='r', bufsize=-1, perms=0664):
     """Use os.open() to open a file in a particular mode.
 
@@ -730,6 +830,8 @@ def open_ex(filename, mode='r', bufsize=-1, perms=0664):
     finally:
         os.umask(ou)
 
+
+
 def GetRequestURI(fallback=None):
     """Return the full virtual path this CGI script was invoked with.
 
